Package: runner
Title: Running Operations for Vectors
Type: Package
Version: 0.4.4
Depends: R (>= 3.0)
Language: en-US
Encoding: UTF-8
Authors@R: person("Dawid", "Kałędkowski", 
                  email = "dawid.kaledkowski@gmail.com",
                  comment = c(ORCID = "0000-0001-9533-457X"),
                  role = c("aut", "cre"))
Maintainer: Dawid Kałędkowski <dawid.kaledkowski@gmail.com>
Description: Lightweight library for rolling windows operations. Package enables
  full control over the window length, window lag and a time indices. With a runner 
  one can apply any R function on a rolling windows. The package eases work with 
  equally and unequally spaced time series.
License: GPL (>= 2)
BugReports: https://github.com/gogonzo/runner/issues
LinkingTo: Rcpp
Imports: methods, parallel, Rcpp
Suggests: knitr, rmarkdown, tinytest
RoxygenNote: 7.3.1
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2024-03-03 20:09:02 UTC; gonzo
Author: Dawid Kałędkowski [aut, cre] (<https://orcid.org/0000-0001-9533-457X>)
Repository: CRAN
Date/Publication: 2024-03-03 20:50:02 UTC
Built: R 4.4.0; x86_64-apple-darwin22.6.0; 2024-05-11 00:13:15 UTC; unix
