# Generated by roxygen2: do not edit by hand

S3method(runner,data.frame)
S3method(runner,default)
S3method(runner,grouped_df)
S3method(runner,matrix)
S3method(runner,xts)
export(fill_run)
export(lag_run)
export(length_run)
export(max_run)
export(mean_run)
export(min_run)
export(minmax_run)
export(run_by)
export(runner)
export(streak_run)
export(sum_run)
export(which_run)
export(window_run)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(parallel,clusterExport)
importFrom(parallel,parLapply)
useDynLib(runner)
